package owl.main.fits;

import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;

import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.JToolBar;

import owl.cameraAPI.CameraAPI;
import owl.gui.utils.OwlButtonFactory;
import owl.main.MainApp;
import owl.main.owltypes.OwlFileChooser;
import owl.main.owltypes.OwlFrame;
import owl.main.owltypes.OwlIconedFileChooser;
import owl.main.owltypes.OwlPanel;
import owl.main.owltypes.OwlTextField;
import owl.main.scripting.ScriptRunnable;



public class FitsHeaderFrame extends OwlFrame
{
	private static final long serialVersionUID = -3303979116717608695L;

	private final String LOAD_ACTION				= "LOAD";
	private final String SAVE_ACTION				= "SAVE";
	private final String EDIT_ACTION				= "EDIT";
//	private final String INFO_ACTION				= "INFO";
//	private final String EDIT_ACTION_1				= "EDIT 1";
//	private final String EDIT_ACTION_2				= "EDIT 2";
//	private final String INFO_ACTION_1				= "INFO 1";
//	private final String INFO_ACTION_2				= "INFO 2";
	private final String CLEAR_ACTION				= "CLEAR";
	private final String INSERT_ACTION				= "INSERT";
	private final String DELETE_ACTION				= "DELETE";
	private final String SCRIPT_BROWSE_ACTION		= "SCRIPT DIR BROWSE";
//	private final String SCRIPT_BROWSE_ACTION_1		= "SCRIPT DIR BROWSE 1";
//	private final String SCRIPT_BROWSE_ACTION_2		= "SCRIPT DIR BROWSE 2";
	private final String XML_FILE_PREF				= "XMLFile";
	private final String SCRIPT_FILE_PREF			= "ScriptFile";
//	private final String SCRIPT_FILE_PREF_1			= "BeforeScriptFile";
//	private final String SCRIPT_FILE_PREF_2			= "AfterScriptFile";
	private final String COMMENT_KEYWORD			= "COMMENT";
	private final String HISTORY_KEYWORD			= "HISTORY";
	private final String DATE_KEYWORD				= "DATE";
	private final String EXPTIME_KEY				= "EXPTIME";
	private final String DATEOBS_KEY				= "DATE-OBS";
	private final String XBINNING_KEY				= "XBINNING";
	private final String YBINNING_KEY				= "YBINNING";

//	private final String FITS_XML_PARSE_RULES	= MainApp.getXMLPath() +
//												  "fits-rules.xml";

	private ArrayList<FitsHeaderElement>	m_fieldList;
	private OwlTextField					m_scriptTxtfld;
	private File							m_scriptFile;
//	private OwlTextField					m_scriptTxtfld_1;
//	private OwlTextField					m_scriptTxtfld_2;
//	private File							m_scriptFile_1;
//	private File							m_scriptFile_2;
	private JPanel							m_panel;
	private File							m_xmlFile;
	private OwlPanel						m_exposePanel;
//	private boolean							m_bStop;

	public FitsHeaderFrame( OwlPanel theExposePanel )
	{
		super( "Fits Header", true );

		m_exposePanel	= theExposePanel;
		m_fieldList		= new ArrayList<FitsHeaderElement>();
		m_scriptFile	= new File( System.getProperty( "user.dir" ) );
//		m_scriptFile_1	= new File( System.getProperty( "user.dir" ) );
//		m_scriptFile_2	= new File( System.getProperty( "user.dir" ) );
//		m_bStop			= false;

		m_panel = new JPanel();
		m_panel.setLayout( new GridLayout( 0, 1 ) );
		m_panel.add( createHeaderTitles() );

		JScrollPane scrollPane = new JScrollPane( m_panel );

		super.addComponent( createToolBar(), super.TOOLBAR_INDEX );
		super.addComponent( scrollPane, super.CENTER_CONTAINER_INDEX );
		super.addComponent( createScriptPanel(), super.SOUTH_CONTAINER_INDEX );

		addField( EXPTIME_KEY, "0.0", "Exposure time" );
		addField( DATEOBS_KEY, "yyyy-mm-ddTHH:MM:SS", "Date and time of observation" );

		pack();

		loadPreferences();
	}

	public void runUpdateScript()
	{
		try
		{
			if ( !m_scriptTxtfld.getText().equals( "" ) )
			{
				//  DO NOT run this script in a thread, as the thread will
				//  compete with the expose thread and cause problems.
				// +------------------------------------------------------+
				String[] varNames = { "fits", "expose" };
				Object[] varObjects = { this, m_exposePanel };

				( new ScriptRunnable( m_scriptTxtfld.getText(), varNames, varObjects ) ).run();
			}
		}
		catch ( Throwable e ) {}
	}

	public void writeToFile( String filename )
	{
		String key = null, val = null, com = null;

		if ( m_fieldList == null ) { return; }

		for ( int i=0; i<m_fieldList.size(); i++ )
		{
			FitsHeaderElement elem = m_fieldList.get( i );

			try
			{
				key = elem.getKey().toUpperCase();
				val = elem.getValue();
				com = elem.getComment();

				if ( key.equals( COMMENT_KEYWORD ) )
				{
					CameraAPI.WriteFitsComment( val, filename );
				}

				else if ( key.equals( HISTORY_KEYWORD ) )
				{
					CameraAPI.WriteFitsHistory( val, filename );
				}

				else if ( key.equals( DATE_KEYWORD ) )
				{
					CameraAPI.WriteFitsDate( filename );
				}

				else if ( !key.equals( "" ) )
				{
					CameraAPI.WriteFitsKeyword( key,
												val,
												com,
												filename );
				}
			}
			catch ( Throwable e )
			{
				MainApp.error( "Failed to write fits keyword. keyword: "
								+ key + " value: " + val
								+ " comment: " + com );
				MainApp.error( e );
			}
		}
	}

	@Override
	public void actionPerformed( ActionEvent ae )
	{
		// |  "Load header from XML" button handler
		// +--------------------------------------------------------------
		if ( ae.getActionCommand().equals( LOAD_ACTION ) )
		{
			String[] ext      = { "xml" };
			String[] desc     = { "XML file ( *.xml )" };
			ImageIcon[] icons = { new ImageIcon( MainApp.getBitmapPath() + "xml.gif" ) };

			OwlIconedFileChooser chooser =
				new OwlIconedFileChooser( new File( System.getProperty( "user.dir" ) ),
										  ext, desc, icons );

			if ( chooser.openDialog() )
			{
				m_xmlFile   = chooser.getSelectedFile();
				m_fieldList = ( new FitsHeaderDigester() ).run( m_xmlFile );

				if ( m_fieldList != null )
				{
					addFieldList();
				}
			}
		}

		// |  "Save header to XML" button handler
		// +--------------------------------------------------------------
		else if ( ae.getActionCommand().equals( SAVE_ACTION ) )
		{
			File file = new File( System.getProperty( "user.dir" ) );
			OwlFileChooser chooser = new OwlFileChooser( file, "xml" );

			if ( chooser.saveDialog() )
			{
				m_xmlFile = chooser.getSelectedFile();
				writeXml( m_xmlFile );
			}
		}

		// |  "Edit update script" button handler
		// +--------------------------------------------------------------
		else if ( ae.getActionCommand().equals( EDIT_ACTION ) )
		{
			m_scriptFile = new File( m_scriptTxtfld.getText() );

			if ( m_scriptFile.isFile() )
			{
				MainApp.launchTextEditor( m_scriptFile.getPath() );
			}
			else
			{
				MainApp.launchTextEditor( "" );

				MainApp.warn(
					"No existing script file. Opening empty text editor." );
			}
		}

//		// |  "Edit update script 1" button handler
//		// +--------------------------------------------------------------
//		else if ( ae.getActionCommand().equals( EDIT_ACTION_1 ) )
//		{
//			m_scriptFile_1 = new File( m_scriptTxtfld_1.getText() );
//
//			if ( m_scriptFile_1.isFile() )
//			{
//				MainApp.launchTextEditor( m_scriptFile_1.getPath() );
//			}
//			else
//			{
//				MainApp.launchTextEditor( "" );
//				MainApp.warn( "No existing script file. Opening empty text editor." );
//			}
//		}
//
//		// |  "Edit update script 2" button handler
//		// +--------------------------------------------------------------
//		else if ( ae.getActionCommand().equals( EDIT_ACTION_2 ) )
//		{
//			m_scriptFile_2 = new File( m_scriptTxtfld_2.getText() );
//
//			if ( m_scriptFile_2.isFile() )
//			{
//				MainApp.launchTextEditor( m_scriptFile_2.getPath() );
//			}
//			else
//			{
//				MainApp.launchTextEditor( "" );
//				MainApp.warn( "No existing script file. Opening empty text editor." );
//			}
//		}
//
//		// |  "Edit update script 1" button handler
//		// +--------------------------------------------------------------
//		else if ( ae.getActionCommand().equals( INFO_ACTION_1 ) )
//		{
//			javax.swing.JOptionPane.showMessageDialog( null,
//						"This script is called BEFORE the exposure starts." );
//		}
//
//		// |  "Edit update script 2" button handler
//		// +--------------------------------------------------------------
//		else if ( ae.getActionCommand().equals( INFO_ACTION_2 ) )
//		{
//			javax.swing.JOptionPane.showMessageDialog( null,
//						"This script is called AFTER the exposure completes." );
//		}

		// |  "Insert header key" button handler
		// +--------------------------------------------------------------
		else if ( ae.getActionCommand().equals( INSERT_ACTION ) )
		{
			if ( m_fieldList != null )
			{
				FitsHeaderElement elem = new FitsHeaderElement();
				elem.setKeyPopupMenuListener( this, DELETE_ACTION );
				m_panel.add( elem );
				m_fieldList.add( elem );
				pack();

				//  Prevent the window from growing indefinitely
				//  off the bottom of the screen.
				// +----------------------------------------------+
				if ( getHeight() > 800 )
				{
					setSize( getWidth(), 700 );
				}
			}
		}

		// |  "Delete header key" button handler
		// +--------------------------------------------------------------
		else if ( ae.getActionCommand().equals( DELETE_ACTION ) )
		{
			JMenuItem src = ( ( JMenuItem )ae.getSource() );

			for ( int i=0; i<m_fieldList.size(); i++ )
			{
				FitsHeaderElement elem = m_fieldList.get( i );

				if ( elem.getIdTag().equals( src.getName() ) )
				{
					m_panel.remove( elem );
					m_fieldList.remove( i );
					pack();
					break;
				}
			}
		}

		// |  "Clear header" button handler
		// +--------------------------------------------------------------
		else if ( ae.getActionCommand().equals( CLEAR_ACTION ) )
		{
			clearAll();
			m_xmlFile = null;
		}

		// |  "Browse update script" button handler
		// +--------------------------------------------------------------
		else if ( ae.getActionCommand().equals( SCRIPT_BROWSE_ACTION ) )
		{
			String[] ext  = { "bsh" };
			String[] desc = { "BeanShell Script ( *.bsh )" };

			ImageIcon[] icons = { new ImageIcon( MainApp.getBitmapPath() +
												 "TinyBean.gif" ) };

			OwlIconedFileChooser fileChooser =
									new OwlIconedFileChooser( m_scriptFile,
															  ext,
															  desc,
															  icons );

			if ( fileChooser.openDialog() )
			{
				m_scriptFile = fileChooser.getSelectedFile();	// Save file for next time
				m_scriptTxtfld.setText( m_scriptFile.getPath() );
			}
		}

//		// |  "Browse update script 1" button handler
//		// +--------------------------------------------------------------
//		else if ( ae.getActionCommand().equals( SCRIPT_BROWSE_ACTION_1 ) )
//		{
//			String[] ext  = { "bsh" };
//			String[] desc = { "BeanShell Script ( *.bsh )" };
//
//			ImageIcon[] icons = { new ImageIcon( MainApp.getBitmapPath() +
//												 "TinyBean.gif" ) };
//
//			OwlIconedFileChooser fileChooser =
//									new OwlIconedFileChooser( m_scriptFile_1,
//															  ext,
//															  desc,
//															  icons );
//
//			if ( fileChooser.openDialog() )
//			{
//				m_scriptFile_1 = fileChooser.getSelectedFile();	// Save file for next time
//				m_scriptTxtfld_1.setText( m_scriptFile_1.getPath() );
//			}
//		}
//
//		// |  "Browse update script 2" button handler
//		// +--------------------------------------------------------------
//		else if ( ae.getActionCommand().equals( SCRIPT_BROWSE_ACTION_2 ) )
//		{
//			String[] ext  = { "bsh" };
//			String[] desc = { "BeanShell Script ( *.bsh )" };
//
//			ImageIcon[] icons = { new ImageIcon( MainApp.getBitmapPath() +
//												 "TinyBean.gif" ) };
//
//			OwlIconedFileChooser fileChooser =
//									new OwlIconedFileChooser( m_scriptFile_2,
//															  ext,
//															  desc,
//															  icons );
//
//			if ( fileChooser.openDialog() )
//			{
//				m_scriptFile_2 = fileChooser.getSelectedFile();	// Save file for next time
//				m_scriptTxtfld_2.setText( m_scriptFile_2.getPath() );
//			}
//		}

		// |  Pass everything else to superclass
		// +--------------------------------------------------------------
		else
		{
			super.actionPerformed( ae );
		}
	}

	public void clearAll()
	{
		if ( m_fieldList != null )
		{
			m_fieldList.clear();
		}

		m_panel.removeAll();
		m_panel.add( createHeaderTitles() );

		pack();
		System.gc();
	}

	public void removeField( String key )
	{
		for ( int i=0; i<m_fieldList.size(); i++ )
		{
			FitsHeaderElement elem = m_fieldList.get( i );

			if ( elem.hasKey( key ) )
			{
				m_fieldList.remove( i );
			}
		}

		m_panel.removeAll();
		m_panel.add( createHeaderTitles() );

		addFieldList();

		pack();

	}

	public boolean hasField( String key )
	{
		Iterator<FitsHeaderElement> listIt = m_fieldList.iterator();

		while ( listIt.hasNext() )
		{
			FitsHeaderElement elem = listIt.next();

			if ( elem.hasKey( key ) )
			{
				return true;
			}
		}

		return false;
	}

	public String getField( String key )
	{
		String value = null;

		Iterator<FitsHeaderElement> listIt = m_fieldList.iterator();

		while ( listIt.hasNext() )
		{
			FitsHeaderElement elem = listIt.next();

			if ( elem.hasKey( key ) )
			{
				value = elem.getValue();
				break;
			}
		}

		return value;
	}

	public void addField( String key, String value )
	{
		addField( key, value, "" );
	}

	public void addField( String key, String value, String comment )
	{
		FitsHeaderElement elem = new FitsHeaderElement();
		elem.setComment( comment );
		elem.setValue( value );
		elem.setKey( key );

		m_fieldList.add( elem );

		addFieldList();
	}

	public void setField( String key, String value )
	{
		setField( key, value, "" );
	}

	public void setField( String key, String value, String comment )
	{
		Iterator<FitsHeaderElement> listIt = m_fieldList.iterator();

		while ( listIt.hasNext() )
		{
			FitsHeaderElement elem = listIt.next();

			if ( elem.hasKey( key ) )
			{
				elem.setComment( comment );
				elem.setValue( value );
			}
		}
	}

	public void setExpTimeField( double gExpTime )
	{
		setField( EXPTIME_KEY, Double.toString( gExpTime ) );
	}

	public void setDateObsField()
	{
		DateFormat dateFormat = new SimpleDateFormat( "yyyy-MM-dd'T'HH:mm:ss" );
 
		setField( DATEOBS_KEY, dateFormat.format( new Date() ) );
	}

	public void setXBinningField( int xFactor )
	{
		if ( hasField( XBINNING_KEY ) )
		{
			setField( XBINNING_KEY, Integer.toString( xFactor ) );
		}
		else
		{
			addField( XBINNING_KEY,
					  Integer.toString( xFactor ),
					  "X axis binning factor" );
		}
	}

	public void removeXBinningField()
	{
		removeField( XBINNING_KEY );
	}

	public void setYBinningField( int yFactor )
	{
		if ( hasField( YBINNING_KEY ) )
		{
			setField( YBINNING_KEY, Integer.toString( yFactor ) );
		}
		else
		{
			addField( YBINNING_KEY,
					  Integer.toString( yFactor ),
					  "Y axis binning factor" );
		}
	}

	public void removeYBinningField()
	{
		removeField( YBINNING_KEY );
	}

	protected JToolBar createToolBar()
	{
		JToolBar toolbar = new JToolBar();
		toolbar.setFloatable( false );

		JButton loadButton = super.createNewToolbarButton( LOAD_ACTION );
		toolbar.add( loadButton );

		JButton saveButton = super.createNewToolbarButton( SAVE_ACTION );
		toolbar.add( saveButton );

		JButton insertButton = super.createNewToolbarButton( INSERT_ACTION );
		toolbar.add( insertButton );

		JButton clearButton = super.createNewToolbarButton( CLEAR_ACTION );
		toolbar.add( clearButton );

		super.appendToolbar( toolbar );

		return toolbar;
	}

	// +--------------------------------------------------------------------+
	// |  savePreferences                                                   |
	// +--------------------------------------------------------------------+
	// |  Saves the xml and script and filenames upon exit                  |
	// +--------------------------------------------------------------------+
	@Override
	protected void savePreferences()
	{
		super.savePreferences();

		String prefKey = null;

		try
		{
			if ( m_xmlFile != null )
			{
				prefKey = getTitle() + XML_FILE_PREF;
				MainApp.getPreferences().put( prefKey, m_xmlFile.getCanonicalPath() );
			}
			else
			{
				prefKey = getTitle() + XML_FILE_PREF;
				MainApp.getPreferences().remove( prefKey );
			}

			if ( m_scriptTxtfld != null && !m_scriptTxtfld.equals( "" ) )
			{
				prefKey = getTitle() + SCRIPT_FILE_PREF;
				MainApp.getPreferences().put( prefKey, m_scriptTxtfld.getText() );
			}
			else
			{
				prefKey = getTitle() + SCRIPT_FILE_PREF;
				MainApp.getPreferences().remove( prefKey );
			}

//			if ( m_scriptTxtfld_1 != null && !m_scriptTxtfld_1.equals( "" ) )
//			{
//				prefKey = getTitle() + SCRIPT_FILE_PREF_1;
//				MainApp.getPreferences().put( prefKey, m_scriptTxtfld_1.getText() );
//			}
//			else
//			{
//				prefKey = getTitle() + SCRIPT_FILE_PREF_1;
//				MainApp.getPreferences().remove( prefKey );
//			}
//
//			if ( m_scriptTxtfld_2 != null && !m_scriptTxtfld_2.equals( "" ) )
//			{
//				prefKey = getTitle() + SCRIPT_FILE_PREF_2;
//				MainApp.getPreferences().put( prefKey, m_scriptTxtfld_2.getText() );
//			}
//			else
//			{
//				prefKey = getTitle() + SCRIPT_FILE_PREF_2;
//				MainApp.getPreferences().remove( prefKey );
//			}
		}
		catch ( IOException ioe ) {}
	}

	// +--------------------------------------------------------------------+
	// |  loadPreferences                                                   |
	// +--------------------------------------------------------------------+
	// |  Loads the xml and script and filenames upon startup               |
	// +--------------------------------------------------------------------+
	@Override
	protected void loadPreferences()
	{
		String prefKey = null;
		String aString = null;

		super.loadPreferences();

		if ( m_scriptTxtfld != null )
		{
			prefKey = getTitle() + XML_FILE_PREF;
			aString = MainApp.getPreferences().get( prefKey, null );
	
			if ( aString != null && !aString.isEmpty() )
			{
				m_xmlFile   = new File( aString );
				m_fieldList = ( new FitsHeaderDigester() ).run( m_xmlFile );
	
				if ( m_fieldList != null )
				{
					addFieldList();
				}
			}

			prefKey = getTitle() + SCRIPT_FILE_PREF;
			aString = MainApp.getPreferences().get( prefKey, null );
	
			if ( aString != null && !aString.isEmpty() )
			{
				try
				{
					m_scriptFile = new File( aString );
	
					if ( m_scriptTxtfld != null && m_scriptFile.isFile() )
					{
						m_scriptTxtfld.setText( m_scriptFile.getCanonicalPath() );
					}
				}
				catch ( IOException ioe ) {}
			}
		}

//		if ( m_scriptTxtfld_1 != null )
//		{
//			prefKey = getTitle() + XML_FILE_PREF;
//			aString = MainApp.getPreferences().get( prefKey, null );
//	
//			if ( aString != null && !aString.isEmpty() )
//			{
//				m_xmlFile   = new File( aString );
//				m_fieldList = readXml( m_xmlFile );
//	
//				if ( m_fieldList != null )
//				{
//					addFieldList();
//				}
//			}
//
//			prefKey = getTitle() + SCRIPT_FILE_PREF_1;
//			aString = MainApp.getPreferences().get( prefKey, null );
//	
//			if ( aString != null && !aString.isEmpty() )
//			{
//				try
//				{
//					m_scriptFile_1 = new File( aString );
//	
//					if ( m_scriptTxtfld_1 != null && m_scriptFile_1.isFile() )
//					{
//						m_scriptTxtfld_1.setText( m_scriptFile_1.getCanonicalPath() );
//					}
//				}
//				catch ( IOException ioe ) {}
//			}
//
//			prefKey = getTitle() + SCRIPT_FILE_PREF_2;
//			aString = MainApp.getPreferences().get( prefKey, null );
//	
//			if ( aString != null && !aString.isEmpty() )
//			{
//				try
//				{
//					m_scriptFile_2 = new File( aString );
//	
//					if ( m_scriptTxtfld_2 != null && m_scriptFile_2.isFile() )
//					{
//						m_scriptTxtfld_2.setText( m_scriptFile_2.getCanonicalPath() );
//					}
//				}
//				catch ( IOException ioe ) {}
//			}
//		}
	}

	private void addFieldList()
	{
		Iterator<FitsHeaderElement> listIt = m_fieldList.iterator();

		while ( listIt.hasNext() )
		{
			FitsHeaderElement elem = listIt.next();
			elem.setKeyPopupMenuListener( this, DELETE_ACTION );
			m_panel.add( elem );
		}

		pack();
	}

	private JPanel createHeaderTitles()
	{
		JPanel panel = new JPanel();
		panel.setLayout( new BoxLayout( panel, BoxLayout.X_AXIS ) );

		JTextField keyTxtfld = new JTextField( "KEY", FitsHeaderElement.KEY_FIELD_WIDTH );
		keyTxtfld.setBackground( new java.awt.Color( 255, 223, 223 ) );
		keyTxtfld.setEnabled( false );
		panel.add( keyTxtfld );

		JTextField valueTxtfld = new JTextField( "VALUE", FitsHeaderElement.VAL_FIELD_WIDTH );
		valueTxtfld.setBackground( new java.awt.Color( 223, 255, 223 ) );
		valueTxtfld.setEnabled( false );
		panel.add( valueTxtfld );

		JTextField commentTxtfld = new JTextField( "COMMENT", FitsHeaderElement.COM_FIELD_WIDTH );
		commentTxtfld.setBackground( new java.awt.Color( 255, 255, 217 ) );
		commentTxtfld.setEnabled( false );
		panel.add( commentTxtfld );

		return panel;
	}

	private JPanel createScriptPanel()
	{
		ImageIcon browseIcon = new ImageIcon( MainApp.getBitmapPath() +
											  "folder.gif",
											  "Browse" );

		JButton browseButton = OwlButtonFactory.create( browseIcon,
														SCRIPT_BROWSE_ACTION,
														new Dimension( 25, 20 ),
														this );

		browseButton.setToolTipText( "Browse for script" );
		
		ImageIcon editIcon = new ImageIcon( MainApp.getBitmapPath() + "edit.gif" );
		
		JButton editButton = OwlButtonFactory.create( editIcon,
													  EDIT_ACTION,
													  new Dimension( 25, 20 ),
													  this );
		
		editButton.setToolTipText( "Edit script" );
		
		m_scriptTxtfld = new OwlTextField( "", 30 );
		
		JPanel panel = new JPanel();
		panel.add( new JLabel( "Update Script: " ) );
		panel.add( m_scriptTxtfld );
		panel.add( editButton );
		panel.add( browseButton );
		

//		JButton browseButton_1 = OwlButtonFactory.create( browseIcon,
//															  SCRIPT_BROWSE_ACTION_1,
//															  new Dimension( 25, 20 ),
//															  this );
//
//		browseButton_1.setToolTipText( "Browse for script" );
//
//		JButton browseButton_2 = OwlButtonFactory.create( browseIcon,
//															 SCRIPT_BROWSE_ACTION_2,
//															 new Dimension( 25, 20 ),
//															 this );
//
//		browseButton_2.setToolTipText( "Browse for script" );
//
//		ImageIcon editIcon = new ImageIcon( MainApp.getBitmapPath() + "edit.gif" );
//
//		JButton editButton_1 = OwlButtonFactory.create( editIcon,
//															EDIT_ACTION_1,
//															new Dimension( 25, 20 ),
//															this );
//
//		editButton_1.setToolTipText( "Edit script" );
//
//		JButton editButton_2 = OwlButtonFactory.create( editIcon,
//														   EDIT_ACTION_2,
//														   new Dimension( 25, 20 ),
//														   this );
//
//		editButton_2.setToolTipText( "Edit script" );
//
//		ImageIcon infoIcon = new ImageIcon( MainApp.getBitmapPath() + "Info1.gif" );
//
//		JButton infoButton_1 = OwlButtonFactory.create( infoIcon,
//														INFO_ACTION_1,
//														new Dimension( 25, 20 ),
//														this );
//
//		JButton infoButton_2 = OwlButtonFactory.create( infoIcon,
//														INFO_ACTION_2,
//														new Dimension( 25, 20 ),
//														this );
//
//		m_scriptTxtfld_1 = new OwlTextField( "", 30 );
//		m_scriptTxtfld_2 = new OwlTextField( "", 30 );
//
//		JPanel panel_1 = new JPanel();
//		panel_1.add( new JLabel( "Update Script 1: " ) );
//		panel_1.add( m_scriptTxtfld_1 );
//		panel_1.add( editButton_1 );
//		panel_1.add( browseButton_1 );
//		panel_1.add( infoButton_1 );
//
//		JPanel panel_2 = new JPanel();
//		panel_2.add( new JLabel( "Update Script 2: " ) );
//		panel_2.add( m_scriptTxtfld_2 );
//		panel_2.add( editButton_2 );
//		panel_2.add( browseButton_2 );
//		panel_2.add( infoButton_2 );

//		JPanel panel = new JPanel();
//		panel.setLayout( new BoxLayout( panel, BoxLayout.Y_AXIS ) );
//		panel.add( panel_1 );
//		panel.add( panel_2 );

		return panel;

//		JPanel panel = new JPanel();
//
//		ImageIcon browseIcon = new ImageIcon( MainApp.getBitmapPath() +
//											  "folder.gif",
//											  "Browse" );
//
//		JButton browseButton = OwlButtonFactory.create( browseIcon,
//														SCRIPT_BROWSE_ACTION,
//													    new Dimension( 25, 20 ),
//													    this );
//
//		browseButton.setToolTipText( "Browse for script" );
//
//		ImageIcon editIcon = new ImageIcon( MainApp.getBitmapPath() +
//											"edit.gif" );
//
//		JButton editButton = OwlButtonFactory.create( editIcon,
//													  EDIT_ACTION,
//													  new Dimension( 25, 20 ),
//													  this );
//		editButton.setToolTipText( "Edit script" );
//
//		scriptTxtfld = new OwlTextField( "", 28 );
//
//		panel.add( new JLabel( "Update Script: " ) );
//		panel.add( scriptTxtfld );
//		panel.add( editButton );
//		panel.add( browseButton );
//
//		return panel;
	}

	private void writeXml( File xmlFile )
	{
		FileWriter fw = null;

		if ( m_fieldList != null )
		{
			try
			{
				if ( !xmlFile.getAbsolutePath().contains( ".xml" ) )
				{
					xmlFile = new File( xmlFile.getAbsolutePath() + ".xml" );
				}

				fw = new FileWriter( xmlFile );
				fw.append( "<?xml version=\"1.0\"?>\n\n" );
				fw.append( "<header>\n" );

				for ( int i=0; i<m_fieldList.size(); i++ )
				{
					FitsHeaderElement elem = m_fieldList.get( i );

					if ( elem.hasAKey() )
					{
						fw.append( "\n\t<card>" );
						fw.append( "\n\t\t<key>" + elem.getKey() + "</key>" );
						fw.append( "\n\t\t<value>" + elem.getValue() + "</value>" );
						fw.append( "\n\t\t<comment>" + elem.getComment() + "</comment>" );
						fw.append( "\n\t</card>\n" );
					}
				}

				fw.append( "\n</header>\n\n" );
				fw.close();
			}
			catch ( Throwable e )
			{
				MainApp.error( e );

				try
				{
					if ( fw != null ) fw.close();
				}
				catch ( Throwable e2 ) {}
			}
		}
	}
}
